@@ -21,15 +21,19 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/syslog.h>
 #include <signal.h>        /* sigaction() */
 #include <pwd.h>           /* pwd entries */
 #include <sys/kd.h>        /* KDGETMODE */
 #include <sys/stat.h>      /* fstat() */
+#include <sys/utsname.h>   /* uname() */
 #include <termios.h>       /* winsize */
 #include <linux/limits.h>  /* OPEN_MAX */
 #include <linux/vt.h>      /* VT_ACTIVATE */
@@ -52,7 +56,9 @@
 #define DEFAULT_BACK 0
 #define DEFAULT_BORD 7
 #define DEFAULT_HEAD 7
-#define usenewscrdump 1 /* should be a variable */
+
+int linux_release;
+#define USE_DEV_VCS (linux_release>1001092)
 
 #if 0
 #define PERROR(s) (fprintf(stderr,"%s: ",prgname),perror(s))
@@ -137,6 +143,9 @@ typedef struct Posted {
 Draw *drawList=NULL;
 
 /* support functions and vars */
+int yyerror(char *s);
+int yylex(void);
+
 DrawItem *cfg_cat(DrawItem *, DrawItem *);
 DrawItem *cfg_makeitem(int mode, char *msg, int(*fun)(), void *detail);
 
@@ -168,7 +177,7 @@ int f_time(int mode, DrawItem *self, int uid);
 int f_pipe(int mode, DrawItem *self, int uid);
 
 
-#line 173 "gpm-root.y"
+#line 182 "gpm-root.y"
 typedef union {
     int silly;
     char *string;
@@ -260,9 +269,9 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   192,   193,   196,   197,   198,   201,   202,   205,   205,   207,
-   208,   209,   210,   211,   214,   214,   216,   218,   219,   222,
-   223,   224
+   201,   202,   205,   206,   207,   210,   211,   214,   214,   216,
+   217,   218,   219,   220,   223,   223,   225,   227,   228,   231,
+   232,   233
 };
 
 static const char * const yytname[] = {   "$","error","$illegal.","T_STRING",
@@ -787,83 +796,83 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 192 "gpm-root.y"
+#line 201 "gpm-root.y"
 {yyval.draw=cfgall=NULL;;
     break;}
 case 2:
-#line 193 "gpm-root.y"
+#line 202 "gpm-root.y"
 {yyvsp[0].draw->buttons=yyvsp[-1].silly; yyvsp[0].draw->next=yyvsp[-2].draw; yyval.draw=cfgall=yyvsp[0].draw;
     break;}
 case 3:
-#line 196 "gpm-root.y"
+#line 205 "gpm-root.y"
 {yyval.silly=GPM_B_LEFT;;
     break;}
 case 4:
-#line 197 "gpm-root.y"
+#line 206 "gpm-root.y"
 {yyval.silly=GPM_B_MIDDLE;;
     break;}
 case 5:
-#line 198 "gpm-root.y"
+#line 207 "gpm-root.y"
 {yyval.silly=GPM_B_RIGHT;;
     break;}
 case 6:
-#line 201 "gpm-root.y"
+#line 210 "gpm-root.y"
 {yyval.draw=cfgcurrent=cfg_alloc();;
     break;}
 case 7:
-#line 202 "gpm-root.y"
+#line 211 "gpm-root.y"
 {yyval.draw=yyvsp[-3].draw; yyval.draw->menu=yyvsp[-1].item;;
     break;}
 case 10:
-#line 207 "gpm-root.y"
+#line 216 "gpm-root.y"
 {cfgcurrent->title=yyvsp[0].string;;
     break;}
 case 11:
-#line 208 "gpm-root.y"
+#line 217 "gpm-root.y"
 {cfgcurrent->back=yyvsp[0].silly;;
     break;}
 case 12:
-#line 209 "gpm-root.y"
+#line 218 "gpm-root.y"
 {cfgcurrent->fore=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 13:
-#line 210 "gpm-root.y"
+#line 219 "gpm-root.y"
 {cfgcurrent->bord=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 14:
-#line 211 "gpm-root.y"
+#line 220 "gpm-root.y"
 {cfgcurrent->head=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 15:
-#line 214 "gpm-root.y"
+#line 223 "gpm-root.y"
 {yyval.silly=0;
     break;}
 case 16:
-#line 214 "gpm-root.y"
+#line 223 "gpm-root.y"
 {yyval.silly=8;
     break;}
 case 17:
-#line 216 "gpm-root.y"
+#line 225 "gpm-root.y"
 {yyval.item=cfg_cat(yyvsp[-1].item,yyvsp[0].item);;
     break;}
 case 18:
-#line 218 "gpm-root.y"
+#line 227 "gpm-root.y"
 {yyval.item=NULL;;
     break;}
 case 19:
-#line 219 "gpm-root.y"
+#line 228 "gpm-root.y"
 {yyval.item= yyvsp[-1].item ? cfg_cat(yyvsp[-1].item,yyvsp[0].item) : yyvsp[0].item;;
     break;}
 case 20:
-#line 222 "gpm-root.y"
+#line 231 "gpm-root.y"
 {yyval.item=cfg_makeitem('F',yyvsp[-1].string,yyvsp[0].fun, NULL);;
     break;}
 case 21:
-#line 223 "gpm-root.y"
+#line 232 "gpm-root.y"
 {yyval.item=cfg_makeitem('2',yyvsp[-2].string,yyvsp[-1].fun, yyvsp[0].string);;
     break;}
 case 22:
-#line 224 "gpm-root.y"
+#line 233 "gpm-root.y"
 {yyval.item=cfg_makeitem('M',yyvsp[-1].string,NULL,yyvsp[0].draw);;
     break;}
 }
@@ -1064,7 +1073,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 227 "gpm-root.y"
+#line 236 "gpm-root.y"
  /* end grammar ###########################################################*/
 
 int yyerror(char *s){
@@ -1120,7 +1129,7 @@ struct funcName funcList[] = {
 };
 
 /*---------------------------------------------------------------------*/
-int yylex()
+int yylex(void)
 {
 int c,i;
 char s[80];
@@ -1151,7 +1160,7 @@ struct funcName *fn;
         s[i-1]=0; yylval.string=(char *)strdup(s); return T_STRING;
 
       case '#':
-        while (c=getc(cfgfile)!='\n' && c!=EOF)
+        while ( (c=getc(cfgfile)!='\n') && c!=EOF)
 	  /* do nothing and fall through */ ;
       case '\n':
         cfglineno++;
@@ -1280,7 +1289,7 @@ static int f_debug_one(FILE *f, Draw *draw)
 {
 DrawItem *ip;
 static int tc=0;
-int i,j;
+int i;
 
 #define LINE(args) for(i=0;i<tc;i++) putc('\t',f); fprintf args
 
@@ -1298,6 +1307,7 @@ int i,j;
       }
     }
 #undef LINE(args)
+  return 0;
 }
 
 int f_debug(int mode, DrawItem *self, int uid)
@@ -1325,8 +1335,6 @@ Draw *dp;
 /*---------------------------------------------------------------------*/
 int f_fgcmd(int mode, DrawItem *self, int uid)
 {
-int i;
-
   switch (mode)
     {
     case F_CREATE:
@@ -1388,8 +1396,10 @@ int i,fd;
       fd=open("/dev/tty0",O_RDWR);
       if (fd<0) {PERROR("/dev/tty0"); return 1;}
       if (ioctl(fd, VT_ACTIVATE, i)<0) {PERROR("ioctl()"); return 1;}
+    default:
       return 0;
     }
+  return 0; /* silly gcc -Wall */
 }
 
 /*---------------------------------------------------------------------*/
@@ -1414,24 +1424,26 @@ char name[10];
 
 	case 0:
 	  /* child: exec getty */
-	  sprintf(name,"tty%i",vc);
+	  sprintf(name,"tty%i",(int)vc);
 	  execl("/sbin/agetty","agetty","38400",name,(char *)NULL);
 	  exit(1); /* shouldn't happen */
 
         default:
 	  /* father: jump to the tty */
-	  sprintf(self->arg,"%i",vc);
+	  sprintf(self->arg,"%i",(int)vc);
 	  return f_jptty(mode,self,uid);
 	}
+    default:
       return 0;
     }
+  return 0;
 }
 
 /*---------------------------------------------------------------------*/
-int f_menu(int mode, DrawItem *self, int uid){}
+int f_menu(int mode, DrawItem *self, int uid){return 0;}
 
 /*---------------------------------------------------------------------*/
-int f_lock(int mode, DrawItem *self, int uid){}
+int f_lock(int mode, DrawItem *self, int uid){return 0;}
 
 /*---------------------------------------------------------------------*/
 int f_load(int mode, DrawItem *self, int uid)
@@ -1635,7 +1647,6 @@ time_t mtime;
 Draw *retrievedraw(int uid, int buttons)
 {
 Draw *drawPtr, *genericPtr=NULL;
-time_t mtime;
 
   /* retrieve a drawing by scanning the list */
   do
@@ -1664,8 +1675,7 @@ int usage(void)
         "Usage: %s [options]\n",prgname);
  printf("  Valid options are\n"
  "    -m <number-or-name>   modifier to use\n"
- "    -B <buffersize>       buffer to embed screendump\n"
- "                              (default is size of current screen)\n"
+ "    -u                    inhibit user configuration files\n"
  );
 
   return 1;
@@ -1713,7 +1723,6 @@ int opt;
     switch (opt)
       {
       case 'm':  opt_mod=getmask(optarg, tableMod); break;
-      case 'B':  opt_buf=atoi(optarg); break;
       case 'u':  opt_user=0;
       default:   return 1;
       }
@@ -1723,33 +1732,82 @@ int opt;
 }
 
 /*------------*/
-static inline void scr_dump(int fd, char *buffer)
+static inline void scr_dump(int fd, FILE *f, char *buffer, int vc)
+    /* WARN */ /* void is no more right */
 {
-if (!usenewscrdump) 
+int dumpfd;
+char dumpname[20];
+
+if (linux_release < 1001068)
   {
   fprintf(stderr,"%s: can't scrdump on kernels < 1.1.68\n",prgname);
   exit(1);
   }
-buffer[0]=8;buffer[1]=0; /* new screendump, fgconsole */
 
-if (ioctl(fd, TIOCLINUX,buffer)<0)
+if (USE_DEV_VCS)
   {
-  PERROR("ioctl(scrdump)");
-  exit(1);
+#if 0
+  FILE *dumpfile; 
+  sprintf(dumpname,"/dev/vcsa%i",vc);
+  dumpfile=fopen(dumpname,"r");
+  if (!dumpfile) return;
+  fread(buffer,1,4,dumpfile);
+  /* /dev/vcs doesn't clear_selection as scrdump used to */
+  GOTOXY(f,buffer[2]+1,buffer[3]+1); fflush(f);
+  fread(buffer+2,1,2*buffer[0]*buffer[1],dumpfile);
+  fclose(dumpfile);
+#else
+  sprintf(dumpname,"/dev/vcsa%i",vc);
+  dumpfd=open(dumpname,O_RDONLY);
+  if (dumpfd<0) return;
+  read(dumpfd,buffer,4);
+  /* /dev/vcs doesn't clear_selection as scrdump used to */
+  GOTOXY(f,buffer[2]+1,buffer[3]+1); fflush(f);
+  read(dumpfd,buffer+4,2*buffer[0]*buffer[1]);
+  close(dumpfd);
+#endif
+  }
+else
+  {
+  buffer[0]=8;buffer[1]=0; /* screendump, fgconsole */
+
+  if (ioctl(fd, TIOCLINUX,buffer)<0)
+    {
+    PERROR("ioctl(scrdump)");
+    exit(1);
+    }
+  buffer[buffer[0]*buffer[1]*2+4]='\0'; /* terminate the string */
   }
-buffer[buffer[0]*buffer[1]*2+4]='\0'; /* terminate the string */
 }
 
 /*------------*/
-static inline void scr_restore(int fd, FILE *f, char *buffer)
+static inline void scr_restore(int fd, FILE *f, char *buffer, int vc)
 {
-int x,y;
-/*
- * GOTOXY(f,0,0);
- * fputs(buffer+2,f);
- */
+int x,y, dumpfd;
+char dumpname[20];
 
-  x=buffer[2]; y=buffer[3];
+x=buffer[2]; y=buffer[3];
+
+if (USE_DEV_VCS)
+  {
+#if 0
+  FILE *dumpfile;
+  sprintf(dumpname,"/dev/vcsa%i",vc);
+  dumpfile=fopen(dumpname,"w");
+  if (!dumpfile) return;
+  fwrite(buffer,1,4+2*buffer[0]*buffer[1],dumpfile);
+  fclose(dumpfile);
+#else
+  sprintf(dumpname,"/dev/vcsa%i",vc);
+  dumpfd=open(dumpname,O_WRONLY);
+  if (dumpfd<0) return;
+  /* GOTOXY(f,x+1,y+1); fflush(f); */
+  write(dumpfd,buffer,4+2*buffer[0]*buffer[1]);
+  close(dumpfd);
+#endif
+  }
+else
+  {
   buffer[0]=9,buffer[1]=0; /* restore, current */
   if (ioctl(fd,TIOCLINUX,buffer)<0)
     {
@@ -1757,6 +1815,7 @@ int x,y;
     exit(1);
     }
   GOTOXY(f,x+1,y+1);
+  }
 }
 
 /*===================================================================*/
@@ -1764,7 +1823,7 @@ int x,y;
 static int postcount;
 static Posted *activemenu;
 
-Posted *postmenu(int fd, FILE *f, Draw *draw, int x, int y)
+Posted *postmenu(int fd, FILE *f, Draw *draw, int x, int y, int console)
 {
 Posted *new;
 DrawItem *item;
@@ -1776,7 +1835,7 @@ short lines,columns;
   if (!new) return NULL;
   new->draw=draw;
   new->dump=dump=malloc(opt_buf);
-  scr_dump(fd,dump);
+  scr_dump(fd,f,dump,console);
   lines=dump[0]; columns=dump[1];
   i=(columns*dump[3]+dump[2])*2+1; /* where to get it */
   if (i<0) i=1;
@@ -1835,11 +1894,11 @@ short lines,columns;
   return new;
   }
 
-Posted *unpostmenu(int fd, FILE *f, Posted *which)
+Posted *unpostmenu(int fd, FILE *f, Posted *which, int vc)
 {
 Posted *prev=which->prev;
 
-  scr_restore(fd,f,which->dump);
+  scr_restore(fd,f,which->dump, vc);
   ioctl(fd,TCXONC,TCOON); /* activate the console */  
   free(which->dump);
   free(which);
@@ -1856,20 +1915,42 @@ pid=wait(&status);
 DEBUG((stderr,"pid %i exited %i\n",pid,status));
 }
 
+
+void get_winsize(void)
+{
+int fd;
+
+  if ((fd=open("/dev/console",O_RDONLY))<0)
+    {
+    fprintf(stderr,"%s: ",prgname); perror("/dev/console");
+    exit(1);
+    }
+  ioctl(fd, TIOCGWINSZ, &win);
+  opt_buf=win.ws_col*win.ws_row;
+  close(fd);
+
+  opt_buf +=4; /* 2:size, 1:terminator, 1:alignment */
+  opt_buf*=2; /* the new scrdump and /dev/vcsa returns color info as well */
+}
+
+
 /*===================================================================*/
+static int do_resize=0;
+void winchHandler(int errno);
+
 int main(int argc, char **argv)
 {
 Gpm_Connect conn;
 Gpm_Event ev;
-int result, vc, fd,uid;
-FILE *f, *ftmp;
+int vc, fd,uid;
+FILE *f;
 struct stat stbuf;
 Draw *draw;
 DrawItem *item;
-char *buffer;
 char s[80];
-int i,j,k, posty, postx, postX;
+int posty, postx, postX;
 struct sigaction childaction;
+struct utsname linux_info;
 
   prgname=argv[0];
 
@@ -1887,6 +1968,17 @@ struct sigaction childaction;
   childaction.sa_mask=0;
   childaction.sa_flags=SA_RESTART;
   sigaction(SIGCHLD,&childaction,NULL);
+  signal(SIGWINCH,winchHandler);
+
+/*....................................... Get version info */
+
+  {
+  int v1,v2,v3;
+  if (uname(&linux_info))
+    {PERROR("uname()"); exit(1);}
+  sscanf(linux_info.release,"%d.%d.%d",&v1,&v2,&v3);
+  linux_release= v1*1000000 + v2*1000 +v3;
+  }
 
 /*....................................... Connect and get your buffer */
 
@@ -1908,26 +2000,8 @@ struct sigaction childaction;
 
   chdir("/");
 
-  if (!opt_buf)
-    {
-    if ((fd=open("/dev/console",O_RDONLY))<0)
-      {
-      fprintf(stderr,"%s: ",prgname); perror("/dev/console");
-      exit(1);
-      }
-    ioctl(fd, TIOCGWINSZ, &win);
-    opt_buf=win.ws_col*win.ws_row;
-    close(fd);
-    }
-  opt_buf +=4; /* 2:size, 1:terminator, 1:alignment */
-  if (usenewscrdump)
-    opt_buf*=2; /* the new scrdump returns color info as well */
 
-#if 0
-  buffer=malloc(opt_buf);
-  if (!buffer)
-    {PERROR("malloc()"); exit(1);}
-#endif
+  get_winsize();
 
 /*....................................... Go to background */
 
@@ -1951,6 +2025,8 @@ struct sigaction childaction;
 
   while(Gpm_GetEvent(&ev)>0)
     {
+    if (do_resize) {get_winsize(); do_resize--;}
+
     /* get rid of spurious events */
     if (ev.type&GPM_MOVE) continue; 
 
@@ -1982,14 +2058,14 @@ struct sigaction childaction;
     uid=stbuf.st_uid;
 
     /* the task now is drawing the box from user data */
-    postmenu(fd,f,draw,ev.x,ev.y);
+    postmenu(fd,f,draw,ev.x,ev.y,vc);
 
     while(Gpm_GetEvent(&ev)>0 && ev.vc==vc)
       {
       Gpm_FitEvent(&ev);
-      Gpm_DrawPointer(ev.x,ev.y,fd);
       if (ev.type&GPM_UP)
 	break; /* we're done */
+      Gpm_DrawPointer(ev.x,ev.y,fd);
       }
     DEBUGG((stderr,"%i - %i\n",posty,ev.y));
 
@@ -1997,10 +2073,18 @@ struct sigaction childaction;
     posty=activemenu->y;
     postx=activemenu->x;
     postX=activemenu->X;
-    unpostmenu(fd,f,activemenu);
+#if 1
+    unpostmenu(fd,f,activemenu,vc);
     close(fd);
     fclose(f);
     Gpm_Close();
+#else /* just to test a bug in /dev/vcs */
+    Gpm_Close();
+    sleep(2);
+    unpostmenu(fd,f,activemenu,vc);
+    close(fd);
+    fclose(f);
+#endif
 
     /* retrieve the item and invoke it */
     posty=ev.y-posty;
@@ -2019,6 +2103,13 @@ struct sigaction childaction;
   exit(0);
 }
 
+void winchHandler(int signo)
+{
+do_resize++;
+signal(SIGWINCH,winchHandler);
+return;
+}
+
 /*
  * This is because Linus uses 4-wide tabstops,
  * forcing me to use the same default to manage kernel sources
@@ -2028,6 +2119,3 @@ struct sigaction childaction;
 /* tab-width:8      */
 /* End:             */
 
-
-
-
