@@ -1,8 +1,8 @@
 /*
  * general purpose mouse support for Linux
  *
- * Copyright 1993   ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -65,6 +65,9 @@ int opt_dirty=DEF_DIRTY;
 int opt_test=DEF_TEST;
 int opt_ptrdrag=DEF_PTRDRAG;
 int opt_kill=0;
+int opt_toggle=DEF_TOGGLE;
+
+static int opt_resize=0; /* not really an option */
 
 char *prgname;
 Gpm_Type *m_type;
@@ -114,7 +117,16 @@ long kd_mode;
     if ((*fdptr=open(opt_dev,O_RDWR))<0)
       oops(opt_dev);
     if (m_type->init)
-      (m_type->init)(*fdptr, m_type->flags, m_type);
+      m_type=(m_type->init)(*fdptr, m_type->flags, m_type);
+
+  if (opt_toggle)
+    {
+    unsigned int modem_lines;
+
+    ioctl(fd, TIOCMGET, &modem_lines);
+    modem_lines &= ~opt_toggle;
+    ioctl(fd, TIOCMSET, &modem_lines);
+    }
   }
 
   return (ch>0);
@@ -162,9 +174,7 @@ int fd;
 /*-------------------------------------------------------------------*/
 static  inline int do_selection(Gpm_Event *event)  /* returns 0, always */
 {
-static int vc=0;
 static int x1=1, y1=1, x2, y2;
-static int ptr=0;
 #define UNPOINTER() 0
 
   x2=event->x; y2=event->y;
@@ -221,7 +231,6 @@ static int ptr=0;
 /* returns 0 if the event has not been processed, and 1 if it has */
 static inline int do_client(Gpm_Cinfo *cinfo, Gpm_Event *event)
 {
-int vc=event->vc;
 Gpm_Connect info=cinfo->data;
 int fd=cinfo->fd;
 /* value to return if event is not used */
@@ -250,7 +259,7 @@ MAGIC_P((static int magic=GPM_MAGIC));
 static inline char *getMouseData(int fd, Gpm_Type *m_type)
 {
 static unsigned char data[32]; /* quite a big margin :) */
-char *edata=data+(m_type->proto)[4];
+char *edata=data+m_type->packetlen;
 int howmany=m_type->howmany;
 int i,j;
 /*....................................... read and identify one byte */
@@ -263,6 +272,11 @@ int i,j;
       }
     if ((data[0]&(m_type->proto)[0]) != (m_type->proto)[1])
       {
+      if (m_type->getextra)
+	{
+	data[1]=GPM_EXTRA_MAGIC_1; data[2]=GPM_EXTRA_MAGIC_2;
+	return data;
+	}
       LOG(("Error in protocol"));
       return NULL;
       }
@@ -274,7 +288,7 @@ int i,j;
      * tried ps2 with the original selection package, which called usleep()
      */
     
-    if (i=(m_type->proto)[4]-howmany) /* still to get */
+    if ( (i=m_type->packetlen-howmany) ) /* still to get */
       do
 	{
 	j=read(fd,edata-i,i); /* edata is pointer just after data */
@@ -301,7 +315,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *m_type)
 {
 char *data;
 static int fine_dx, fine_dy;
-static int i, margin, j;
+static int i, margin;
 static Gpm_Event nEvent;
 static struct vt_stat stat;
 static int clicks=0;    /* 0,1 or 2 */
@@ -407,6 +421,7 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
     case GPM_MOVE:
       clicks=0;
+    default:
       break;
     }
   event->clicks=clicks;
@@ -429,7 +444,7 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
   event->margin=margin;
 
-  LOGG(("M: %3i %3i (%3i %3i) - butt=%i vc=%i cl=%i",event->dx,event->dy,
+  LOG(("M: %3i %3i (%3i %3i) - butt=%i vc=%i cl=%i",event->dx,event->dy,
                                       event->x,event->y,
                                       event->buttons, event->vc,
                                       event->clicks));
@@ -442,7 +457,7 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 /*-------------------------------------------------------------------*/
 static inline int get_data(Gpm_Connect *where, int whence)
 {
-static int check, i;
+static int i;
 
 #ifdef GPM_USE_MAGIC
   while ((i=read(whence,&check,sizeof(int)))==4 && check!=GPM_MAGIC)
@@ -470,8 +485,7 @@ static int check, i;
                                  /* returns -1 if closing connection */
 static inline int processRequest(int fd, int vc) 
 {
-int i, check;
-Gpm_Connect request;
+int i;
 Gpm_Cinfo *cinfoPtr, *next;
 
   LOG(("Request on %i (console %i)",fd,vc));
@@ -517,7 +531,7 @@ static inline int processConn(int fd) /* returns newfd or -1 */
 Gpm_Cinfo *info;
 Gpm_Connect *request;
 Gpm_Cinfo *next;
-int check, vc, newfd, len;
+int vc, newfd, len;
 struct sockaddr_un addr; /* reuse this each time */
 
 /*....................................... Accept */
@@ -570,14 +584,30 @@ struct sockaddr_un addr; /* reuse this each time */
 
   return newfd;
 }
+
+/*-------------------------------------------------------------------*/
+void get_console_size(Gpm_Event *ePtr)
+{
+int i;
+  i=open_console(O_RDONLY);
+  ioctl(i, TIOCGWINSZ, &win);
+  close(i);
+  if (!win.ws_col || !win.ws_row)
+    {
+    fprintf(stderr, "%s: zero screen dimension, assuming 80x25.\n",prgname);
+    win.ws_col=80; win.ws_row=25;
+    }
+  maxx=win.ws_col; maxy=win.ws_row;
+  DEBUGG((stderr,"%i - %i\n",maxx,maxy));
+  ePtr->x=maxx/2; ePtr->y=maxy/2;
+}
+
 /*-------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-FILE *f;
-struct sockaddr_un addr[MAX_VC+1];
 int mousefd, ctlfd, newfd;
 struct sockaddr_un ctladdr;
-int pid, i, len;
+int i, len;
 struct   timeval timeout;
 int maxfd=-1;
 int pending;
@@ -589,9 +619,10 @@ Gpm_Event event;
   mousefd=cmdline(argc, argv);
   maxfd=max(mousefd,maxfd);
 
-  signal(SIGTERM,gpm_killed);
-  signal(SIGUSR1,gpm_killed); /* usr1 is used by a new gpm killing the old */
-  signal(SIGINT, gpm_killed);
+  signal(SIGTERM, gpm_killed);
+  signal(SIGINT,  gpm_killed);
+  signal(SIGUSR1, gpm_killed); /* usr1 is used by a new gpm killing the old */
+  signal(SIGWINCH,gpm_killed); /* winch can be sent if console is resized */
 
 /*....................................... create your nodes */
 
@@ -612,16 +643,7 @@ Gpm_Event event;
 
 /*....................................... get screen dimensions */
 
-  i=open_console(O_RDONLY);
-  ioctl(i, TIOCGWINSZ, &win);
-  close(i);
-  if (!win.ws_col || !win.ws_row)
-    {
-    fprintf(stderr, "%s: zero screen dimension, assuming 80x25.\n",prgname);
-    win.ws_col=80; win.ws_row=25;
-    }
-  maxx=win.ws_col; maxy=win.ws_row;
-  event.x=maxx/2; event.y=maxy/2;
+  get_console_size(&event);
   
 /*....................................... wait for mouse and connections */
 
@@ -662,6 +684,17 @@ Gpm_Event event;
 	resetTimeout();
 	} /* go on */
 
+    if (opt_resize)
+      {
+      get_console_size(&event);
+      opt_resize--;
+      signal(SIGWINCH,gpm_killed); /* reinstall handler */
+
+      for(i=0; i<MAX_VC+1; i++)
+	if (cinfo[i])
+	  kill(cinfo[i]->data.pid,SIGWINCH);
+      }
+
     if (pending<0)
       {
       LOG(("select(): %s",strerror(errno)));
@@ -733,6 +766,12 @@ Gpm_Event event;
 /*-------------------------------------------------------------------*/
 static void gpm_killed(int signo)
 {
+  if (signo==SIGWINCH)
+    {
+    DEBUGG((stderr,"%s pid %i is resizing :-)\n",prgname,getpid()));
+    opt_resize++;
+    return;
+    }
   if (signo==SIGUSR1)
     fprintf(stderr,"%s pid %i killed by a new %s\n",prgname,getpid(),prgname);
   else
