@@ -1,7 +1,7 @@
 /*
  * gpm-Linux configuration file (server only)
  *
- * Copyright 1994   rubini@ipvvis.unipv.it
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
  ********/
 
 #define GPM_NAME    "gpm-Linux"
-#define GPM_RELEASE "0.97"
-#define GPM_DATE    "January 1995"
+#define GPM_RELEASE "0.98"
+#define GPM_DATE    "March 1995"
 
 /* You can define DEBUG, DEBUGG, or neither. */
 /* Well, better putting them in the Makefile... */
@@ -43,7 +43,7 @@
 /* all the default values */
 #define DEF_TYPE          "ms"
 #define DEF_DEV           NULL     /* use the type-related one */
-#define DEF_LUT   "a-zA-Z0-9_./-"  /* this is my personal taste... */
+#define DEF_LUT   "-a-zA-Z0-9_./\300-\326\330-\366\370-\377"
 #define DEF_SEQUENCE     "123"     /* how buttons are reordered */
 #define DEF_BAUD          1200
 #define DEF_SAMPLE         100
@@ -57,4 +57,4 @@
 #define DEF_DIRTY            0
 #define DEF_TEST             0
 #define DEF_PTRDRAG         10    /* never */
-
+#define DEF_TOGGLE           0
