@@ -1,7 +1,7 @@
 /*
  * gpmInt.h - server-only stuff for gpm-Linux
  *
- * Copyright 1994   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,10 +32,10 @@
 #  define LOG(args) Log args
 extern int Log(char *fmt, ...);
 #else
-#  define LOG(args) 1
+#  define LOG(args) 
 #endif
 /* a placeholder */
-#define LOGG(args) 1
+#define LOGG(args) 
 
 /*....................................... Structures */
 
@@ -46,12 +46,17 @@ extern int Log(char *fmt, ...);
 typedef struct Gpm_Type {
   char *name;
   int (*fun)(Gpm_Event *state, unsigned char *data);
-  int (*init)(int fd, unsigned short flags, struct Gpm_Type *type);
+  struct Gpm_Type *(*init)(int fd, unsigned short flags,
+						   struct Gpm_Type *type);
   unsigned short flags;
-  unsigned char proto[5];
+  unsigned char proto[4];
+  int packetlen;
   int howmany; /* howmany bytes to read at a time */
+  int getextra; /* does it get an extra byte? (only mouseman) */
 }                   Gpm_Type;
 
+#define GPM_EXTRA_MAGIC_1 0xAA
+#define GPM_EXTRA_MAGIC_2 0x55
 
 typedef struct Gpm_Cinfo {
   Gpm_Connect data;
@@ -75,7 +80,7 @@ extern char *opt_type, *opt_lut, *opt_dev;
 extern char *opt_sequence;
 extern int opt_baud, opt_sample, opt_delta, opt_accel, opt_scale;
 extern int opt_dirty, opt_test, opt_time, opt_cluster, opt_three, opt_ptrdrag;
-extern int opt_kill;
+extern int opt_kill, opt_toggle;
 
 extern Gpm_Type mice[];
 extern Gpm_Type *m_type;
@@ -95,11 +100,6 @@ int cmdline(int argc, char **argv);
 
 /*....................................... Dirty hacks */
 
-#ifndef linux /* no strerror() on sunos 4.1 */
-extern char *sys_errlist[];
-#define strerror(foo) (sys_errlist[foo])
-#endif
-
 #undef GPM_USE_MAGIC /* magic token foreach message? */
 
 
