@@ -1,8 +1,8 @@
 /*
  * gpn.c - support functions for gpm-Linux
  *
- * Copyright 1993   ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 #include <signal.h>
 #include <stdarg.h>        /* Log uses it */
 #include <errno.h>
-#include <unistd.h>        /* getopt() */
+#include <unistd.h>        /* getopt(),symlink() */
 #include <sys/stat.h>      /* mkdir()  */
 #include <sys/param.h>
 #include <sys/time.h>      /* timeval */
@@ -90,38 +90,70 @@ int _oops(char *f, int n, char *s, int err)
 }
 
 /*===================================================================*/
+/* octal digit */
+static int isodigit(const unsigned char c)
+{
+  return ((c & ~7) == '0');
+}
+
+/* routine to convert digits from octal notation (Andries Brouwer) */
+static int getsym(const unsigned char *p0, unsigned char *res)
+{
+  const unsigned char *p = p0;
+  char c;
+
+  c = *p++;
+  if (c == '\\' && *p) 
+    {
+    c = *p++;
+    if (isodigit(c)) 
+      {
+      c -= '0';
+      if (isodigit(*p)) c = 8*c + (*p++ - '0');
+      if (isodigit(*p)) c = 8*c + (*p++ - '0');
+      }
+    }
+  *res = c;
+  return (p - p0);
+}
+
 static int loadlut(char *charset)
 {
-int i, c, fd;
-static unsigned long long_array[5]={
-  0x05050505,      /* ugly, but preserves alignment */
-  0x0,0x0,0x0,0x0
-};
+  int i, c, fd;
+  unsigned char this, next;
+  static unsigned long long_array[9]={
+    0x05050505, /* ugly, but preserves alignment */
+    0x00000000, /* control chars     */
+    0x00000000, /* digits            */
+    0x00000000, /* uppercase and '_' */
+    0x00000000, /* lowercase         */
+    0x00000000, /* Latin-1 control   */
+    0x00000000, /* Latin-1 misc      */
+    0x00000000, /* Latin-1 uppercase */
+    0x00000000  /* Latin-1 lowercase */
+    };
+
 
 #define inwordLut (long_array+1)
-#define SET(c) (inwordLut[((c)>>5)&3] |= (1<<((c)&0x1F)))
 
-   for (i=0; (c=charset[i]); i++)
-     {
-     if (c=='\\')    /* use the next, if any */
-       {
-       if (charset[i+1]=='\0') break;
-       c=charset[++i];
-       SET(c);
-       }
-    else if (c=='-' && i!=0 && charset[i+1]!='\0') /* range */
-      for (c=charset[i-1]; c<charset[i+1]; c++) SET(c);
-    else SET(c);
+  for (i=0; charset[i]; )
+    {
+    i += getsym(charset+i, &this);
+    if (charset[i] == '-' && charset[i + 1] != '\0')
+      i += getsym(charset+i+1, &next) + 1;
+    else
+      next = this;
+    for (c = this; c <= next; c++)
+      inwordLut[c>>5] |= 1 << (c&0x1F);
     }
-
+  
   if ((fd=open("/dev/console", O_WRONLY)) < 0)
     oops("/dev/console");
   if ((fd<0) || (ioctl(fd, TIOCLINUX, &long_array)<0))
     {
-    if (getuid())
-      fprintf(stderr,"%s: you're not root, can you ioctl(TIOCLINUX)?\n",
-	      prgname);
-    else
+    if (errno==EPERM && getuid())
+      fprintf(stderr,"%s: you should probably be root\n",prgname);
+    else if (errno==EINVAL)
       fprintf(stderr,"%s: is your kernel compiled with CONFIG_SELECTION on?\n",
 	      prgname);
     oops("loadlut");
@@ -147,6 +179,7 @@ static int usage(void)
  "    -l charset       loads the inword() LUT (default '%s')\n"
  "    -L charset       load LUT and exit\n"
  "    -m mouse-device  sets mouse device\n"
+ "    -o modem-lines   toggle modem lines (\"dtr\", \"rts\", \"both\")\n"
  "    -p               draw the pointer while striking a selection\n"
  "    -P               do it only with word and line drags\n"
  "    -r number        sets the responsiveness (default %i)\n"
@@ -155,6 +188,7 @@ static int usage(void)
  "                     Microsoft = `ms', Mouse Systems Corp = `msc',\n"
  "                     MM Series = `mm', Logitech = `logi', BusMouse = `bm',\n"
  "                     MSC 3-bytes = `sun', PS/2 = `ps2')\n"
+ "                     MouseMan = `mman', oldest 2-button serial = `bare'\n"
  "    -T               test: read mouse, no clients\n"
  "    -v               print version info and exit\n",
         DEF_ACCEL, DEF_BAUD, DEF_SEQUENCE, DEF_DELTA, DEF_TIME, DEF_LUT,
@@ -165,7 +199,7 @@ static int usage(void)
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dhi:kl:L:m:pPr:s:t:Tv23";
+char options[]="a:b:B:d:Dhi:kl:L:m:o:pPr:s:t:Tv23";
 int opt, fd;
 static struct {char *in; char *out;} seq[] = {
   {"123","01234567"},
@@ -192,6 +226,11 @@ static struct {char *in; char *out;} seq[] = {
       case 'l': opt_lut=optarg; break;
       case 'L': loadlut(optarg); exit(0);
       case 'm': opt_dev = optarg; break;
+      case 'o':
+          if (!strcmp(optarg,"dtr"))       opt_toggle=TIOCM_DTR;
+          else if (!strcmp(optarg,"dtr"))  opt_toggle=TIOCM_RTS;
+          else if (!strcmp(optarg,"both")) opt_toggle=TIOCM_DTR | TIOCM_RTS;
+          else exit(usage());
       case 'p': opt_ptrdrag=0; break;
       case 'P': opt_ptrdrag=1; break;
       case 'r':
@@ -207,9 +246,7 @@ static struct {char *in; char *out;} seq[] = {
       case '2': opt_three=-1; break;
       case '3': opt_three=1; break;
       default:
-          fprintf(stderr,"%s: try \"%s -h\" to get help\n",
-		  prgname, prgname);
-	  exit(1);
+          exit(usage());
       }
     }
 
@@ -225,15 +262,7 @@ static struct {char *in; char *out;} seq[] = {
     exit(usage());
 
 
-
-  /* check for uniqueness and sign */
-
-  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
-    oops(GPM_NODE_DIR);
-  if (chdir(GPM_NODE_DIR))
-    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
-
-  /* this is the new authentication */
+  /* check for uniqueness */
   {
   struct sockaddr_un ctladdr;
   int ctlfd, pid, len; FILE *f;
@@ -272,8 +301,9 @@ static struct {char *in; char *out;} seq[] = {
 
   close(ctlfd);
 
-  /* go to background and become a session leader (Stefan Giessler) */
+#ifndef DEBUGGING
 
+  /* go to background and become a session leader (Stefan Giessler) */
   switch(fork())
     {
     case -1: oops("fork()"); /* error */
@@ -281,12 +311,35 @@ static struct {char *in; char *out;} seq[] = {
     default: exit(0);        /* parent */
     }
 
-  if (!freopen("/dev/console","w",stderr))
+  if (!freopen("/dev/console","w",stderr)) /* the currently current console */
     {
     printf("gpm: freopen(stderr) failed\n"); exit(1);
     }
   if (setsid()<0) oops("setsid()");
 
+#endif
+
+  /* open the device */
+  if (opt_dev)
+    {
+    if (!strcmp(opt_dev,"-"))
+      fd=0;
+    else if ((fd=open(opt_dev,O_RDWR))<0)
+      oops(opt_dev); /* user failed */
+    }
+  else /* use "/dev/mouse" */
+    {
+    opt_dev = "/dev/mouse";
+    if ((fd=open(opt_dev,O_RDWR))<0)
+      oops("/dev/mouse");
+    }
+
+  /* chdir */
+  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
+    oops(GPM_NODE_DIR);
+  if (chdir(GPM_NODE_DIR))
+    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
+
   /* now sign */
   f=fopen(GPM_NODE_PID,"w");
   if (!f)
@@ -300,23 +353,25 @@ static struct {char *in; char *out;} seq[] = {
   LOG(("Signed"));
   }
 
-  /* open the device */
-  if (opt_dev)
-    {
-    if ((fd=open(opt_dev,O_RDWR))<0)
-      oops(opt_dev); /* user failed */
-    }
-  else /* use "/dev/mouse" */
-    {
-    opt_dev = "/dev/mouse";
-    if ((fd=open(opt_dev,O_RDWR))<0)
-      oops("/dev/mouse");
-    }
+  /*
+   * well, now create a symlink in the /tmp dir to be compliant with old
+   * executables
+   */
+  symlink(GPM_NODE_CTL,"/tmp/gpmctl");
 
-  /* init the device */
+  /* init the device, and change mouse type */
   if (m_type->init)
-    (m_type->init)(fd, m_type->flags, m_type);
+    m_type=(m_type->init)(fd, m_type->flags, m_type);
 
+  if (opt_toggle)
+    {
+    unsigned int modem_lines;
+
+    ioctl(fd, TIOCMGET, &modem_lines);
+    modem_lines &= ~opt_toggle;
+    ioctl(fd, TIOCMSET, &modem_lines);
+    }
+ 
   loadlut(opt_lut);
 
   /* choose the sequence */
