@@ -1,7 +1,7 @@
 /*
  * liblow.c - client library - low level (gpm-Linux)
  *
- * Copyright 1994   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
  * 
  * xterm management is mostly by Miguel de Icaza
  *
@@ -22,6 +22,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>        /* strncmp */
 #include <unistd.h>        /* select(); */
 #include <sys/time.h>      /* timeval */
@@ -112,7 +113,7 @@ Gpm_Stst *new;
     gpm_flag=1;
     return gpm_fd;
     }
-/*....................................... No xterm, go on *
+/*....................................... No xterm, go on */
 
 
 
@@ -329,7 +330,7 @@ static int count;
 #define DELAY_MS 100
     static struct timeval to={0,DELAY_MS*1000};
     static fd_set selSet;
-    static int prevchar=EOF, clicks=0;
+    static int prevchar=EOF;
     extern gpm_convert_event(char *data, Gpm_Event *event);
     int c; char mdata[4];
 
