@@ -28,8 +28,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <curses.h> /* to set raw mode */
 
 #include "gpmCfg.h"  /* need the config one to get GPM_NAME */
@@ -92,7 +95,7 @@ return 0;
 /*-------------------------------------------------------------------*/
 int emacs_handler(Gpm_Event *event, void *data)
 {
-int i,j; char *evs;
+int i,j;
 static int dragX, dragY;
 static char buffer[64];
 static char *s_mod[]={"S-","M-","C-","M-",NULL};
@@ -278,11 +281,18 @@ char *argv[20];
   return 0;
 }
 
+void sigHandler(int signo)
+{
+/* just propagate to the father */
+  kill(getppid(),SIGWINCH);
+  signal(SIGWINCH,sigHandler);
+}
+
 /*===================================================================*/
 int main(int argc, char **argv)
 {
 Gpm_Connect conn;
-int c,d,flags;
+int c,d;
 char s[8];
 char cmd[128];
 
@@ -294,6 +304,8 @@ char cmd[128];
   if (opt_pointer)
     opt_pointer=open("/dev/console",O_RDWR); /* can't be 0 */
 
+  signal(SIGWINCH,sigHandler);
+
 /*....................................... Init before printing */
 
   if (!opt_intrct && isatty(1))
@@ -310,7 +322,7 @@ char cmd[128];
 
 if (!opt_silent)
   fprintf(stderr,"This program prints on stdout any event it"
-	  "gets from stdin and from the mouse\n");
+	  "gets from stdin and from the mouse\r\n");
 
   conn.eventMask=opt_mask;
   conn.defaultMask=opt_default;
@@ -337,6 +349,10 @@ if (!opt_silent)
 
   else
     {
+	int modifiers; char modstring[32];
+	printf("Well, the modifiers below are only valid for virtual consoles\r\n"
+		   "\tbut I'm too lazy to check it\r\n");
+
     while ((c=Gpm_Getc(stdin))!=EOF)
       {
       if (isatty(0) && (c==4)) break; /* ctl-D to exit */
@@ -345,8 +361,22 @@ if (!opt_silent)
       strcat(s, c<' ' ? "C-" : "");
       if (c<' ') d+=0x40;
       if (c==0x7F) d='?';
-      printf("key: %02X ('%s%c')\r\n",c,s,d);
-      }
+
+	  modifiers=6; /* code for the ioctl */
+	  if (ioctl(0,TIOCLINUX,&modifiers)<0)
+		modifiers=0;
+	  modstring[0]=0;
+	  if (modifiers&1) strcat(modstring,", shift");
+	  if (modifiers&2) strcat(modstring,", right-alt");
+	  if (modifiers&4) strcat(modstring,", control");
+	  if (modifiers&8) strcat(modstring,", left-alt");
+	  if (!modifiers)  strcpy(modstring,", none");
+
+      printf("key-modifiers: %02X-%02X \t'%s%c'\t%s\r\n",
+			 c,modifiers,s,d,modstring+2);
+	  }
+
+
     if (isatty(1) && !opt_intrct)
       {
       noraw();
