@@ -50,7 +50,10 @@
 #include "gpmInt.h"
 
 /*========================================================================*/
-/* Ok, here we are: first, provide the functions */
+/*
+ * Ok, here we are: first, provide the functions
+ * The return value is the number of unprocessed bytes
+ */
 
 static int M_ms(Gpm_Event *state,  unsigned char *data)
 {
@@ -60,7 +63,7 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
    */
   static unsigned char prev=0;
 
-#if 1
+#if 0 /* original patch */
 
   if (data[0] == 0x40 && !(prev|data[1]|data[2]))
     state->buttons = 2;           /* third button on MS compatible mouse */
@@ -93,6 +96,15 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
+static int M_bare(Gpm_Event *state,  unsigned char *data)
+{
+  /* a bare ms protocol */
+  state->buttons= ((data[0] & 0x20) >> 3) | ((data[0] & 0x10) >> 4);
+  state->dx=      (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
+  state->dy=      (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+  return 0;
+}
+
 static int M_sun(Gpm_Event *state,  unsigned char *data)
 {
   state->buttons= (~data[0]) & 0x07;
@@ -148,6 +160,47 @@ static int M_ps2(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
+static int M_mman(Gpm_Event *state,  unsigned char *data)
+{
+  /*
+   * the damned MouseMan has 3/4 bytes packets. The extra byte 
+   * is only there if the middle button is active.
+   * I get the extra byte as a packet with magic numebers in it.
+   * and then switch to 4-byte mode.
+   */
+  static unsigned char prev=0;
+  static Gpm_Type *mytype=mice; /* it is the first */
+
+  if (data[1]==GPM_EXTRA_MAGIC_1 && data[2]==GPM_EXTRA_MAGIC_2)
+	{
+	/* got unexpected fourth byte */
+	LOG(("Extra byte = %02x",*data));
+	if (*data != 0x20) return -1; /* must be 0x20 */
+	state->buttons= (prev |= GPM_B_MIDDLE);
+	state->dx=state->dy=0;
+
+	mytype->packetlen=4;
+	mytype->getextra=0;
+	}
+  else
+	{
+	/* got 3/4, as expected */
+	state->buttons= ((data[0] & 0x20) >> 3) | ((data[0] & 0x10) >> 4);
+	if (mytype->packetlen==4)
+	  {
+	  if (data[3]) state->buttons |= GPM_B_MIDDLE;
+	  else {mytype->packetlen=3; mytype->getextra=1;}
+	  }
+	}
+
+  /* motion is independent of packetlen... */
+  state->dx= (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
+  state->dy= (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+  prev=state->buttons;
+  
+  return 0;
+}
+
 /*========================================================================*/
 /* Then, mice should be initialized */
 
@@ -207,11 +260,10 @@ static struct {
     {1E9,"N"},
 };
 
-static int I_serial(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_serial(int fd, unsigned short flags, struct Gpm_Type *type)
 {
-int i;
-fd_set set;
-struct timeval timeout={0,0};
+int i; unsigned char c;
+fd_set set; struct timeval timeout={0,0}; /* used when not debugging */
 
 
   /* change from any available speed to the chosen one */
@@ -225,23 +277,33 @@ struct timeval timeout={0,0};
   write(fd,sampletab[i].code,1);
 #endif
 
-  /* flush any pending input (thanks, Miguel) */
+#ifndef DEBUGGING
+  /*
+   * flush any pending input (thanks, Miguel)
+   * moreover, damned mouseman's can be detected here (though it isn't)
+   */
   FD_ZERO(&set);
-  while (1)
+  for(i=0; /* always */ ; i++)
 	{
 	FD_SET(fd,&set);
 	switch(select(fd+1,&set,(fd_set *)NULL,(fd_set *)NULL,&timeout/* zero */))
 	  {
-	  case 1:  read(fd,&i,1);
+	  case 1:  if (read(fd,&c,1)==0) break;
 	  case -1: continue;
 	  }
 	break;
 	}
+#endif
 
-  return 0;
+  if (type->fun==M_ms && i==2 && c==0x33) /* Aha.. a mouseman... */
+	{
+	LOG(("MouseMan detected"));
+	return mice; /* it is the first */
+	}
+  return type;
 }
 
-static int I_logi(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_logi(int fd, unsigned short flags, struct Gpm_Type *type)
 {
 int i;
 struct stat buf;
@@ -268,7 +330,7 @@ int busmouse;
   for (i=0;opt_sample<=sampletab[i].sample;i++)
     ;
   write(fd,sampletab[i].code,1);
-  return 0;
+  return type;
 }
 
 
@@ -277,22 +339,26 @@ int busmouse;
 #define STD_FLG (CREAD|CLOCAL|HUPCL)
 
 Gpm_Type mice[]={
+  {"mman", M_mman, I_serial, CS7 | STD_FLG, /* MUST BE FIRST */
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 1},
   {"ms",   M_ms, I_serial, CS7 | STD_FLG,
-                                {0x40, 0x40, 0x40, 0x00, 3}, 1},
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0},
+  {"bare", M_bare, I_serial, CS7 | STD_FLG,
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0},
   {"sun",  M_sun, I_serial, CS8 | CSTOPB | STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00, 3}, 1},
+                                {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0},
   {"msc",  M_msc, I_serial, CS8 | CSTOPB | STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00, 5}, 1},
+                                {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0},
   {"mm",   M_mm, I_serial, CS8 | PARENB|PARODD | STD_FLG,
-                                {0xe0, 0x80, 0x80, 0x00, 3}, 1},
+                                {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0},
   {"logi", M_logi, I_logi, CS8 | CSTOPB | STD_FLG,
-                                {0xe0, 0x80, 0x80, 0x00, 3}, 3},
+                                {0xe0, 0x80, 0x80, 0x00}, 3, 3, 0},
   {"bm",   M_bm, NULL,  STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00, 3}, 3},
+                                {0xf8, 0x80, 0x00, 0x00}, 3, 3, 0},
   {"ps2",  M_ps2, NULL,  STD_FLG,
-                                {0xc0, 0x00, 0x00, 0x00, 3}, 1},
+                                {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0},
 
-  {"",     NULL, NULL, 0,   {0x00, 0x00, 0x00, 0x00, 0}}
+  {"",     NULL, NULL, 0,       {0x00, 0x00, 0x00, 0x00}, 0, 0, 0}
 };
 
 
