@@ -48,8 +48,8 @@
 #endif
 
 #define RMEV_NAME "Mev"
-#define RMEV_RELEASE "0.97"
-#define RMEV_DATE "December 1994"
+#define RMEV_RELEASE "0.98"
+#define RMEV_DATE "March 1995"
 
 
 /*===================================================================*/
