@@ -63,7 +63,7 @@ by 't-mouse.el")
 	 (delete-process t-mouse-process))
     (setq t-mouse-process 
 	  ;; use only press events
-	  (start-process "t-mouse" nil "mev" "-E" "-C" (t-mouse-tty) "-M" "0"
+	  (start-process "t-mouse" nil "mev" "-E" "-C" (t-mouse-tty) "-M" "leftAlt"
 			 "-e" "press,drag,release,hard" "-d" "move"))
     (set-process-filter t-mouse-process 't-mouse-process-filter)
     (process-kill-without-query t-mouse-process)
@@ -148,6 +148,9 @@ by 't-mouse.el")
 	(double-mouse-1 .    tm-word)
 	(triple-mouse-1 .    tm-line)
 	(double-mouse-3 .    tm-kill)
+	(M-down-mouse-1 .    tm-scroll-up)
+	(M-down-mouse-2 .    tm-scroll-jump)
+	(M-down-mouse-3 .    tm-scroll-down)
 	(mode-line keymap
 		   (mouse-1 . tm-win-select)
 		   (mouse-2 . tm-win-single)
@@ -157,6 +160,7 @@ by 't-mouse.el")
 		    (down-mouse-3   . tm-scroll-down)
 		    (down-mouse-2   . tm-scroll-jump)
 		    (drag-mouse-2   . tm-scroll-drag)
+		    (M-drag-mouse-2 . tm-scroll-drag)
 		    (mouse-1        . tm-scroll-done)
 		    (double-mouse-1 . tm-scroll-done)
 		    (triple-mouse-1 . tm-scroll-done)
@@ -165,7 +169,16 @@ by 't-mouse.el")
 		    (triple-mouse-2 . tm-scroll-done)
 		    (mouse-3        . tm-scroll-done)
 		    (double-mouse-3 . tm-scroll-done)
-		    (triple-mouse-3 . tm-scroll-done))))
+		    (triple-mouse-3 . tm-scroll-done)
+		    (M-mouse-1        . tm-scroll-done)
+		    (M-double-mouse-1 . tm-scroll-done)
+		    (M-triple-mouse-1 . tm-scroll-done)
+		    (M-mouse-2        . tm-scroll-done)
+		    (M-double-mouse-2 . tm-scroll-done)
+		    (M-triple-mouse-2 . tm-scroll-done)
+		    (M-mouse-3        . tm-scroll-done)
+		    (M-double-mouse-3 . tm-scroll-done)
+		    (M-triple-mouse-3 . tm-scroll-done))))
 
 ;;; first, the scrollbar
 
